"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const util = require("util");
const utility_1 = require("../utility");
const Project_1 = require("./Project");
class ProjectQueue {
    constructor() {
        this.changes = new Map();
    }
}
class Client {
    constructor(id, placeId) {
        this.id = id;
        this.placeId = placeId;
        this.sendQueue = new Map();
        Client._instances.push(this);
        this.fullSyncAllToStudio();
    }
    getProjectQueue(projectName) {
        let projectQueue = this.sendQueue.get(projectName);
        if (projectQueue === undefined) {
            projectQueue = new ProjectQueue();
            this.sendQueue.set(projectName, projectQueue);
        }
        return projectQueue;
    }
    fullSyncProjectToStudio(project) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!project.loaded) {
                console.log("not loaded", project.name);
                return;
            }
            const promises = new Array();
            yield Promise.all(project.partitions.map(partition => partition.addChangesRecursive(promises)));
            const changes = yield Promise.all(promises);
            this.getProjectQueue(project.name).initial = true;
            this.syncToStudio(project.name, changes);
        });
    }
    fullSyncAllToStudio() {
        return __awaiter(this, void 0, void 0, function* () {
            Project_1.default.instances
                .filter(project => project.isValidPlaceId(this.placeId))
                .forEach(project => this.fullSyncProjectToStudio(project));
        });
    }
    remove() {
        this.disconnect();
        const index = Client.instances.indexOf(this);
        if (index > -1) {
            Client._instances.splice(index, 1);
        }
    }
    writeResponse() {
        if (this.response) {
            const payload = new Array();
            for (const projectName of this.sendQueue.keys()) {
                const projectQueue = this.sendQueue.get(projectName);
                if (projectQueue === undefined) {
                    continue;
                }
                const changes = new Array();
                for (const key of projectQueue.changes.keys()) {
                    const change = projectQueue.changes.get(key);
                    if (change) {
                        changes.push(change);
                        projectQueue.changes.delete(key);
                    }
                }
                const initial = projectQueue.initial;
                if (changes.length > 0 || initial === true) {
                    payload.push({ projectName, changes, initial });
                    projectQueue.initial = false;
                }
            }
            if (payload.length > 0) {
                console.log("send", payload
                    .map(value => util.format("%s%s (%d)", value.projectName, value.initial ? "*" : "", value.changes ? value.changes.length : 0))
                    .join(", "), util.format("[%d]", this.placeId));
                const res = this.response;
                this.response = undefined;
                utility_1.writeJson(res, payload);
            }
        }
    }
    setResponse(res) {
        this.disconnect();
        this.response = res;
        this.writeResponse();
    }
    syncToStudio(projectName, changes) {
        const projectQueue = this.getProjectQueue(projectName);
        for (const change of changes) {
            const changePath = [...change.path];
            if (changePath[changePath.length - 1] === "init") {
                changePath.pop();
            }
            projectQueue.changes.set(changePath.join("/") + "/" + change.type, change);
        }
        this.writeResponse();
    }
    disconnect(res) {
        if (this.response && (res === undefined || res === this.response)) {
            this.response.end();
            this.response = undefined;
        }
    }
}
Client._instances = new Array();
Client.instances = Client._instances;
exports.default = Client;
//# sourceMappingURL=Client.js.map