"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const http = require("http");
const util = require("util");
const anymatch = require("anymatch");
const utility_1 = require("../utility");
class Server {
    constructor() {
        this.routes = new Array();
        this.enabled = true;
        this.httpServer = http.createServer((req, res) => {
            const desiredRoute = req.url;
            let didRespond = false;
            if (desiredRoute && this.enabled) {
                const validRoutes = this.routes.filter(route => route.method === req.method);
                for (const route of validRoutes) {
                    if (anymatch(route.glob, desiredRoute)) {
                        try {
                            route.callback(req, res);
                        }
                        catch (e) {
                            if (e instanceof Error) {
                                utility_1.writeError(res, e.message);
                            }
                        }
                        didRespond = true;
                        return;
                    }
                }
            }
            if (!didRespond) {
                res.statusCode = 404;
                res.end();
            }
        });
    }
    listen(port) {
        if (this.httpServer) {
            this.httpServer.listen(port).on("error", err => {
                if (err.errno === "EADDRINUSE") {
                    console.log("Failed to use port 8888. Is Rofresh already running?");
                    process.exit(1);
                }
                else {
                    throw err;
                }
            });
        }
    }
    close() {
        if (this.httpServer) {
            this.httpServer.close();
            this.httpServer = undefined;
        }
    }
    registerRoute(glob, callback, method) {
        const routeGlobExists = this.routes.reduce((accum, route) => accum || (route.glob === glob && route.method === method), false);
        if (routeGlobExists) {
            throw new Error(util.format("Route already exists! [ %s %s ]", method, glob));
        }
        this.routes.push({ glob, callback, method });
    }
    get(glob, callback) {
        this.registerRoute(glob, callback, "GET");
    }
    post(glob, callback) {
        this.registerRoute(glob, callback, "POST");
    }
}
exports.default = Server;
//# sourceMappingURL=Server.js.map