#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const jsLog = console.log;
console.log = (...args) => jsLog("\x1b[31m[Rofresh]\x1b[0m", ...args);
const fs = require("mz/fs");
const util = require("util");
const commander = require("commander");
const rofresh = require("./rofresh");
const io_ts = require("io-ts");
const DEFAULT_PROJECT_DIR = ".";
(() => __awaiter(this, void 0, void 0, function* () {
    const pkgVersion = io_ts.string.decode(require("../package.json").version).getOrElse("N/A");
    const projectFolders = new Array();
    commander
        .version(pkgVersion, "-v, --version")
        .arguments("[dir...]")
        .option("-i --install [dir]", "Install Studio Plugin Automatically")
        .action((folders) => {
        for (const folder of folders) {
            projectFolders.push(folder);
        }
    })
        .parse(process.argv);
    if (commander.install) {
        let installDir = commander.install;
        if (typeof installDir === "string") {
            if (!(yield fs.exists(installDir))) {
                throw new Error(util.format("Specified installation directory does not exist! [ %s ]", installDir));
            }
        }
        else {
            installDir = null;
        }
        const result = yield rofresh.installPlugin(installDir);
        if (result === rofresh.PluginInstallResult.Success) {
            console.log("Rofresh plugin successfully installed!");
        }
        else if (result === rofresh.PluginInstallResult.Failure) {
            console.log("Rofresh plugin could not be installed automatically!");
        }
        else if (result === rofresh.PluginInstallResult.PromptRestartStudio) {
            console.log("Rofresh plugin successfully installed!");
            console.log("Please restart Roblox Studio.");
        }
    }
    else {
        if (projectFolders.length === 0) {
            rofresh.addProject(DEFAULT_PROJECT_DIR);
        }
        else {
            for (const dir of projectFolders) {
                if (yield fs.exists(dir)) {
                    rofresh.addProject(dir);
                }
            }
        }
        rofresh.start();
    }
}))();
//# sourceMappingURL=exec.js.map