"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const Client_1 = require("./class/Client");
const Language_1 = require("./class/Language");
const Project_1 = require("./class/Project");
const Server_1 = require("./class/Server");
const utility_1 = require("./utility");
const PORT = 8888;
const server = new Server_1.default();
server.enabled = false;
function getClient(req) {
    const clientId = req.headers["client-id"];
    const placeIdStr = req.headers["roblox-id"];
    if (!clientId || typeof clientId !== "string") {
        throw new Error("Bad clientId!");
    }
    if (!placeIdStr || typeof placeIdStr !== "string") {
        throw new Error("Bad placeId!");
    }
    const placeId = parseInt(placeIdStr, 10);
    if (typeof placeId !== "number" || isNaN(placeId)) {
        throw new Error("placeId must be a number!");
    }
    let client = Client_1.default.instances.filter(value => value.id === clientId)[0];
    if (client) {
        if (client.placeId !== placeId) {
            client.placeId = placeId;
            client.fullSyncAllToStudio();
        }
    }
    else {
        client = new Client_1.default(clientId, placeId);
    }
    return client;
}
server.get("/", (req, res) => getClient(req).setResponse(res));
server.get("/projects", (req, res) => {
    const client = getClient(req);
    utility_1.writeJson(res, Project_1.default.instances.filter(project => project.isValidPlaceId(client.placeId)).map(project => project.name));
});
server.get("/debug", (req, res) => {
    utility_1.writeJsonDebug(res, {
        clients: Client_1.default.instances,
        languages: Language_1.default.instances,
        projects: Project_1.default.instances,
    });
});
try {
    server.listen(PORT);
}
catch (e) {
    console.log("e", e);
}
var utility_2 = require("./utility");
exports.installPlugin = utility_2.installPlugin;
exports.PluginInstallResult = utility_2.PluginInstallResult;
/**
 * determines if rofresh is currently running
 */
function isRunning() {
    return server.enabled;
}
exports.isRunning = isRunning;
/**
 * adds a project directory to rofresh
 */
function addProject(dir) {
    dir = path.resolve(dir);
    if (!Project_1.default.instances.some(p => p.directory === dir)) {
        const project = new Project_1.default(dir);
        if (server.enabled) {
            project.start();
        }
    }
}
exports.addProject = addProject;
/**
 * removes a project directory from rofresh
 */
function removeProject(dir) {
    dir = path.resolve(dir);
    Project_1.default.instances.filter(project => project.directory === dir).forEach(project => {
        project.stop();
        project.remove();
    });
}
exports.removeProject = removeProject;
/**
 * stops rofresh
 */
function stop() {
    if (server.enabled) {
        console.log("stop", "server");
        server.enabled = false;
        Project_1.default.instances.forEach(project => project.stop());
        Client_1.default.instances.forEach(client => client.remove());
    }
}
exports.stop = stop;
/**
 * starts rofresh
 */
function start() {
    if (!server.enabled) {
        console.log("start", "server");
        server.enabled = true;
        Project_1.default.instances.forEach(project => project.start());
    }
}
exports.start = start;
//# sourceMappingURL=rofresh.js.map