"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process = require("mz/child_process");
const fs = require("mz/fs");
const path = require("path");
const PLUGIN_FILE_NAME = "RofreshPlugin.rbxmx";
const ROBLOX_STUDIO_PROCESS_NAME = "RobloxStudioBeta";
const MAX_FILE_RETRY = 5;
const FILE_RETRY_DELAY = 10; // ms
const LATEST_PLUGIN_VERSION = 1;
function decircularJson(object) {
    let cache = new Array();
    const result = JSON.stringify(object, (_, value) => {
        if (typeof value === "object" && value !== null) {
            if (cache.indexOf(value) !== -1) {
                try {
                    return JSON.parse(JSON.stringify(value));
                }
                catch (error) {
                    return;
                }
            }
            cache.push(value);
        }
        return value;
    });
    cache = null;
    return result;
}
function writeJsonDebug(res, object) {
    res.setHeader("content-type", "application/json");
    res.write(decircularJson(object));
    res.end();
}
exports.writeJsonDebug = writeJsonDebug;
function writeJson(res, object) {
    res.setHeader("content-type", "application/json");
    res.write(JSON.stringify(object));
    res.end();
}
exports.writeJson = writeJson;
function writeError(res, errorMsg) {
    writeJson(res, { error: errorMsg });
}
exports.writeError = writeError;
function wait(ms) {
    return new Promise(resolve => setTimeout(() => resolve(), ms));
}
exports.wait = wait;
function getFileContents(filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        let attempt = 0;
        let fileContents;
        do {
            attempt++;
            fileContents = yield fs.readFile(filePath);
            // hack!
            if (fileContents.length === 0 && attempt <= MAX_FILE_RETRY) {
                yield wait(FILE_RETRY_DELAY);
            }
        } while (fileContents.length === 0 && attempt <= MAX_FILE_RETRY);
        return fileContents;
    });
}
exports.getFileContents = getFileContents;
const WIN32_CMD = "tasklist";
const WIN32_SUFFIX = ".exe";
const DARWIN_CMD = "ps -ax | grep ";
function isProcessRunning(name) {
    return __awaiter(this, void 0, void 0, function* () {
        let cmd = "";
        if (process.platform === "win32") {
            cmd = WIN32_CMD;
            name += WIN32_SUFFIX;
        }
        else if (process.platform === "darwin") {
            cmd = DARWIN_CMD + name;
        }
        else {
            return false;
        }
        const [stdout, stderr] = yield child_process.exec(cmd);
        return (stdout
            .toString()
            .toLowerCase()
            .indexOf(name.toLowerCase()) > -1);
    });
}
exports.isProcessRunning = isProcessRunning;
function getPluginFolderWin32() {
    return __awaiter(this, void 0, void 0, function* () {
        const appData = process.env.LOCALAPPDATA;
        if (appData) {
            const robloxFolder = path.join(appData, "Roblox");
            if (yield fs.exists(robloxFolder)) {
                const pluginsFolder = path.join(robloxFolder, "Plugins");
                if (!(yield fs.exists(pluginsFolder))) {
                    yield fs.mkdir(pluginsFolder);
                }
                return pluginsFolder;
            }
        }
    });
}
function getPluginFolderDarwin() {
    return __awaiter(this, void 0, void 0, function* () {
        return undefined;
    });
}
var PluginInstallResult;
(function (PluginInstallResult) {
    PluginInstallResult[PluginInstallResult["Success"] = 0] = "Success";
    PluginInstallResult[PluginInstallResult["Failure"] = 1] = "Failure";
    PluginInstallResult[PluginInstallResult["PromptRestartStudio"] = 2] = "PromptRestartStudio";
    PluginInstallResult[PluginInstallResult["AlreadyInstalled"] = 3] = "AlreadyInstalled";
})(PluginInstallResult = exports.PluginInstallResult || (exports.PluginInstallResult = {}));
/**
 * attempts to automatically install the Rofresh Roblox Studio plugin
 */
function installPlugin(installDir) {
    return __awaiter(this, void 0, void 0, function* () {
        const pluginPath = path.join(__dirname, "..", PLUGIN_FILE_NAME);
        if (!(yield fs.exists(pluginPath))) {
            throw new Error("Plugin file missing!");
        }
        let pluginFolder;
        if (installDir) {
            pluginFolder = path.join(installDir, "Plugins", PLUGIN_FILE_NAME);
        }
        else {
            if (process.platform === "win32") {
                pluginFolder = yield getPluginFolderWin32();
            }
            else if (process.platform === "darwin") {
                pluginFolder = yield getPluginFolderDarwin();
                // TODO
                return PluginInstallResult.Failure;
            }
            else {
                return PluginInstallResult.Failure;
            }
        }
        if (pluginFolder === undefined) {
            return PluginInstallResult.Failure;
        }
        const pluginFileExt = path.extname(PLUGIN_FILE_NAME);
        const pluginFileBaseName = path.basename(PLUGIN_FILE_NAME, pluginFileExt);
        const installPath = path.join(pluginFolder, pluginFileBaseName + "_" + LATEST_PLUGIN_VERSION + pluginFileExt);
        let currentFound = false;
        for (const fileName of yield fs.readdir(pluginFolder)) {
            const filePath = path.join(pluginFolder, fileName);
            const matches = fileName.match(new RegExp("^" + PLUGIN_FILE_NAME + "_(\\d+)$"));
            if (!matches) {
                continue;
            }
            const version = parseInt(matches[1], 10);
            if (!isNaN(version)) {
                if (!currentFound && version >= LATEST_PLUGIN_VERSION) {
                    currentFound = true;
                }
                else {
                    fs.unlink(filePath);
                }
            }
        }
        if (currentFound) {
            return PluginInstallResult.Success;
        }
        // copy
        yield fs.writeFile(installPath, yield fs.readFile(pluginPath));
        if (yield isProcessRunning(ROBLOX_STUDIO_PROCESS_NAME)) {
            return PluginInstallResult.PromptRestartStudio;
        }
        else {
            return PluginInstallResult.Success;
        }
    });
}
exports.installPlugin = installPlugin;
//# sourceMappingURL=utility.js.map